﻿namespace Tiobon.Core.Services.BASE;

/// <summary>
/// 增删改查基础服务
/// </summary>
/// <typeparam name="TEntity"></typeparam>
/// <typeparam name="TEntityDto"></typeparam>
/// <typeparam name="TInsertDto"></typeparam>
/// <typeparam name="TEditDto"></typeparam>
public class BaseServices<TEntity, TEntityDto, TInsertDto, TEditDto> : IBaseServices<TEntity, TEntityDto, TInsertDto, TEditDto> where TEntity : class, new()
{
    public ICaching _caching;

    public BaseServices(IBaseRepository<TEntity> BaseDal = null, ICaching caching = null)
    {
        this.BaseDal = BaseDal;
        _caching = caching;
    }

    //public IBaseRepository<TEntity> baseDal = new BaseRepository<TEntity>();
    public IBaseRepository<TEntity> BaseDal { get; set; } //通过在子类的构造函数中注入，这里是基类，不用构造函数

    public ISqlSugarClient Db => BaseDal.Db;
    public int? LangId => App.User.GetLangId();

    /// <summary>
    /// 根据ID查询实体数据是否存在
    /// </summary>
    /// <param name="objId"></param>
    /// <returns></returns>
    public async Task<bool> AnyAsync(object objId)
    {
        var data = await BaseDal.AnyAsync(objId);
        return data;
    }

    public async Task<bool> AnyAsync(Expression<Func<TEntity, bool>> whereExpression)
    {
        var data = await BaseDal.AnyAsync(whereExpression);
        return data;
    }

    public virtual async Task<TEntityDto> QueryById(object objId)
    {
        var data = new TEntity();
        var dto = Mapper.Map(data).ToANew<TEntityDto>();

        var entityInfo = Db.EntityMaintenance.GetEntityInfo(typeof(TEntity));

        var keyName = entityInfo.Columns.Where(x => x.IsPrimarykey == true).FirstOrDefault()?.DbColumnName ?? "Id";

        string sql = @$"DECLARE @langId   INT = 1,@ID BIGINT = '{objId}';
                SELECT *,
                         isnull
                         ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                           FROM Ghrs_User kk
                           WHERE kk.UserId = a.CreateBy),
                          '')
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000078', @langId, '于 ')
                       + ' '
                       + CONVERT (NVARCHAR (16), CreateTime, 121)
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000079', @langId, ' 创建')
                          CreateDataInfo,
                         isnull
                         ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                           FROM Ghrs_User kk
                           WHERE kk.UserId = a.UpdateBy),
                          '')
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000078', @langId, '于')
                       + ' '
                       + CONVERT (NVARCHAR (16), UpdateTime, 121)
                       + ' '
                       + [dbo].[FLangKeyToValue]
                         ('GHR_Common_000080', @langId, ' 最后修改')
                          UpdateDataInfo
                FROM {entityInfo.DbTableName} a
                WHERE a.{keyName} = @ID AND IsEnable='1'";
        dto = await Db.Ado.SqlQuerySingleAsync<TEntityDto>(sql);
        return dto;
    }

    /// <summary>
    /// 根据ID查询一条数据
    /// </summary>
    /// <param name="objId">id（必须指定主键特性 [SugarColumn(IsPrimaryKey=true)]），如果是联合主键，请使用Where条件</param>
    /// <param name="blnUseCache">是否使用缓存</param>
    /// <returns>数据实体</returns>
    public async Task<TEntityDto> QueryById(object objId, bool blnUseCache = false)
    {
        var data = await BaseDal.QueryById(objId, blnUseCache);
        return Mapper.Map(data).ToANew<TEntityDto>();
    }

    /// <summary>
    /// 根据ID查询数据
    /// </summary>
    /// <param name="lstIds">id列表（必须指定主键特性 [SugarColumn(IsPrimaryKey=true)]），如果是联合主键，请使用Where条件</param>
    /// <returns>数据实体列表</returns>
    public async Task<List<TEntityDto>> QueryByIDs(object[] lstIds)
    {
        var data = await BaseDal.QueryByIDs(lstIds);
        return Mapper.Map(data).ToANew<List<TEntityDto>>();
    }

    public virtual async Task<ServiceFormResult<TEntityDto>> QueryForm(QueryForm body)
    {
        var data = new ServiceFormResult<TEntityDto>();
        string sql = string.Empty;
        var dt = new DataTable();

        if (body.id != null)
        {
            Type entityType = typeof(TEntity);
            sql = @$"DECLARE @langId   INT = '{body.langId}',@ID BIGINT = '{body.id}';

                SELECT *,
                         isnull
                         ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                           FROM Ghrs_User kk
                           WHERE kk.UserId = a.CreateBy),
                          '')
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000078', @langId, '于 ')
                       + ' '
                       + CONVERT (NVARCHAR (16), CreateTime, 121)
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000079', @langId, ' 创建')
                          CreateDataInfo,
                         isnull
                         ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                           FROM Ghrs_User kk
                           WHERE kk.UserId = a.UpdateBy),
                          '')
                       + ' '
                       + [dbo].[FLangKeyToValue] ('GHR_Common_000078', @langId, '于')
                       + ' '
                       + CONVERT (NVARCHAR (16), UpdateTime, 121)
                       + ' '
                       + [dbo].[FLangKeyToValue]
                         ('GHR_Common_000080', @langId, ' 最后修改')
                          UpdateDataInfo
                FROM {entityType.GetEntityTableName()} a
                WHERE a.Id = @ID";
            data.result.DT_TableDataT1 = await Db.Ado.SqlQueryAsync<TEntityDto>(sql);
        }
        else
        {
            var list = new List<TEntity>
            {
                new TEntity()
            };

            data.result.DT_TableDataT1 = Mapper.Map(list).ToANew<List<TEntityDto>>();

        }
        #region JM_PageFormActionsT1
        dynamic JM_PageFormActionsT1 = new ExpandoObject();

        //var toolbars = await _caching.GetAsync<List<FormToolbar>>(body.menuName + "FormToolbar");
        //if (toolbars == null || (toolbars != null && !toolbars.Any()))
        //{

        //    if (toolbars.Any())
        //        await _caching.SetAsync(body.menuName + "FormToolbar", toolbars);
        //}
        sql = @$"SELECT field fnKey, [dbo].[FLangKeyToValue] (mkey, {body.langId}, label) fnTitle, icon
                FROM Ghrs_PageSettingEdit
                WHERE IsEnable = 1 AND pageNo = '{body.menuName}' AND elementType = 'FnKey'";
        var toolbars = await Db.Ado.SqlQueryAsync<FormToolbar>(sql);

        JM_PageFormActionsT1.Toolbar = toolbars;
        #endregion

        #region JM_TableColumnT1
        dynamic JM_TableColumnT1 = new ExpandoObject();
        var tableColumn = new JArray();
        sql = @$"SELECT field,
                       [dbo].[FLangKeyToValue] (mkey, {body.langId}, label)
                          label,
                       required,
                       editable,
                       rowNum,
                       colNum,
                       dataType,
                       elementType,
                       dbo.FS_GetdataSourceBySet
                       (dataSource, APIDataSourceType, Ghrs_PageSettingEdit.APIDataSourceID)
                          dataSource,
                       defaultHidden,
                       isPrimaryKey,
                       isSingleColumn,multipleSelect
                FROM Ghrs_PageSettingEdit
                WHERE     IsEnable = 1
                      AND pageNo = '{body.menuName}'
                      AND elementType NOT IN ('FnKey', 'PageGroup');";
        dt = await Db.Ado.GetDataTableAsync(sql);

        for (int i = 0; i < dt.Rows.Count; i++)
        {
            JObject item =
                new JObject {
                    new JProperty("field", dt.Rows[i]["field"].ToString()),
                    new JProperty("label", dt.Rows[i]["label"].ToString()),
                    new JProperty("required", !string.IsNullOrWhiteSpace(dt.Rows[i]["required"].ToString()) ? Convert.ToBoolean(dt.Rows[i]["required"]) : null),
                    new JProperty("editable", !string.IsNullOrWhiteSpace(dt.Rows[i]["editable"].ToString()) ? Convert.ToBoolean(dt.Rows[i]["editable"]) : null),
                    new JProperty("rowNum", !string.IsNullOrWhiteSpace(dt.Rows[i]["rowNum"].ToString()) ? Convert.ToInt32(dt.Rows[i]["rowNum"]) : null),
                    new JProperty("colNum", !string.IsNullOrWhiteSpace(dt.Rows[i]["colNum"].ToString()) ? Convert.ToInt32(dt.Rows[i]["colNum"]) : null),
                    new JProperty("dataType", dt.Rows[i]["dataType"].ToString()),
                    new JProperty("elementType", dt.Rows[i]["elementType"].ToString()),
                    new JProperty("dataSource", dt.Rows[i]["dataSource"].ToString()),
                    new JProperty("defaultHidden", dt.Rows[i]["defaultHidden"].ToString()),
                    new JProperty("isPrimaryKey", !string.IsNullOrWhiteSpace(dt.Rows[i]["isPrimaryKey"].ToString()) ? Convert.ToBoolean(dt.Rows[i]["isPrimaryKey"]) : null),
                    new JProperty("isSingleColumn", !string.IsNullOrWhiteSpace(dt.Rows[i]["isSingleColumn"].ToString()) ? Convert.ToInt32(dt.Rows[i]["isSingleColumn"]) : null),
                    new JProperty("multipleSelect", !string.IsNullOrWhiteSpace(dt.Rows[i]["multipleSelect"].ToString()) ? Convert.ToBoolean(dt.Rows[i]["multipleSelect"]) : null)
                };
            tableColumn.Add(item);
        }
        JM_TableColumnT1.TableColumn = tableColumn;
        #endregion

        #region JM_PageFormT1
        dynamic JM_PageFormT1 = new ExpandoObject();
        var pageForm = new JArray();
        JObject pageFormItem = new JObject {
            new JProperty("disabled", false),
            new JProperty("labelAlign", "right"),
            new JProperty("labelCol", "{span: 6}"),
            new JProperty("labelWidth",120),
            new JProperty("layout", "horizontal")
        };

        sql = $@"SELECT [dbo].[FLangKeyToValue] (MKey, {body.langId}, MenuName) pageTitle,
                       IUDProcedure apiName
                FROM Ghrs_Menu
                WHERE MenuNo = '{body.menuName}' AND IsEnable = 1";
        dt = await Db.Ado.GetDataTableAsync(sql);
        if (dt.Rows.Count > 0)
        {
            pageFormItem.Add(new JProperty("pageTitle", dt.Rows[0]["pageTitle"].ToString()));
            pageFormItem.Add(new JProperty("apiName", dt.Rows[0]["apiName"].ToString()));
        }
        pageForm.Add(pageFormItem);
        JM_PageFormT1.PageForm = pageForm;
        #endregion

        #region DT_PageMutiMsg
        var DT_PageMutiMsg = await _caching.GetAsync<List<DT_PageMutiMsg>>("DT_PageMutiMsg");
        if (DT_PageMutiMsg == null || (DT_PageMutiMsg != null && !DT_PageMutiMsg.Any()))
        {
            sql = $@"SELECT Langkey field,
                       CASE {body.langId}
                          WHEN 1 THEN isnull (Value01, LangValue)
                          WHEN 2 THEN isnull (Value02, LangValue)
                          WHEN 3 THEN isnull (Value03, LangValue)
                          WHEN 4 THEN isnull (Value04, LangValue)
                          WHEN 5 THEN isnull (Value05, LangValue)
                          WHEN 6 THEN isnull (Value06, LangValue)
                          WHEN 7 THEN isnull (Value07, LangValue)
                          WHEN 8 THEN isnull (Value08, LangValue)
                          WHEN 9 THEN isnull (Value09, LangValue)
                          WHEN 10 THEN isnull (Value10, LangValue)
                       END label
                FROM Ghrs_LangKey
                WHERE     (LangKey LIKE 'GHR_Page%' OR LangKey LIKE 'GHR_Common%')
                      AND IsEnable = 1";
            DT_PageMutiMsg = Db.Ado.SqlQuery<DT_PageMutiMsg>(sql);
            if (DT_PageMutiMsg.Any())
                await _caching.SetAsync("DT_PageMutiMsg", DT_PageMutiMsg);
        }
        #endregion

        data.result.JM_PageFormActionsT1 = JM_PageFormActionsT1;
        data.result.JM_TableColumnT1 = JM_TableColumnT1;
        data.result.JM_PageFormT1 = JM_PageFormT1;
        data.result.DT_PageMutiMsg = DT_PageMutiMsg;
        return data;
    }

    #region 新增

    /// <summary>
    /// 写入实体数据
    /// </summary>
    /// <param name="entity">实体类</param>
    /// <returns></returns>
    public virtual async Task<long> Add(TInsertDto entity)
    {
        var entity1 = Mapper.Map(entity).ToANew<TEntity>();
        if (UserContext.Context != null)
        {
            HttpRequest request = UserContext.Context.Request;
            var api = request.Path.ObjToString().TrimEnd('/').ToLower();
            var ip = GetUserIp(UserContext.Context);

            BasePoco ent = entity1 as BasePoco;

            ent.CreateIP = ip;
            ent.CreateProg = api;
        }


        #region 检查是否存在相同值
        CheckOnly(entity1);
        #endregion

        return await BaseDal.Add(entity1);
    }

    /// <summary>
    /// 批量插入实体(速度快)
    /// </summary>
    /// <param name="listEntity">实体集合</param>
    /// <returns>影响行数</returns>
    public virtual async Task<List<long>> Add(List<TInsertDto> listEntity)
    {
        if (listEntity != null && listEntity.Any())
        {
            var userId = App.User.ID;
            var api = string.Empty;
            var ip = string.Empty;
            if (UserContext.Context != null)
            {
                HttpRequest request = UserContext.Context.Request;
                api = request.Path.ObjToString().TrimEnd('/').ToLower();
                ip = GetUserIp(UserContext.Context);
            }


            var list = new List<TEntity>();

            foreach (var item in listEntity)
            {
                var entity = new TEntity();
                ConvertTInsertDto2TEntity(item, entity);
                list.Add(entity);
            }

            //var list = Mapper.Map(listEntity).ToANew<List<TEntity>>();
            list.ForEach(entity =>
            {
                BasePoco ent = entity as BasePoco;
                ent.CreateIP = ip;
                ent.CreateProg = api;


                #region 检查是否存在相同值
                CheckOnly(entity);
                #endregion
            });
            return await BaseDal.Add(list);
        }
        else return default;
    }
    #endregion

    /// <summary>
    /// 更新实体数据
    /// </summary>
    /// <param name="entity">实体类</param>
    /// <returns></returns>
    public virtual async Task<bool> Update(long Id, TEditDto editModel)
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);

        if (editModel == null || !await BaseDal.AnyAsync(Id))
            return false;

        var entity = await BaseDal.QueryById(Id);
        ConvertTEditDto2TEntity(editModel, entity);
        BasePoco ent = entity as BasePoco;
        ent.UpdateIP = ip;
        ent.UpdateProg = api;
        CheckOnly(entity, Id);
        return await BaseDal.Update(entity);
    }
    public virtual async Task<bool> Update(long Id, TEditDto editModel, List<string> lstColumns = null, List<string> lstIgnoreColumns = null, string where = "")
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);

        if (editModel == null || !await BaseDal.AnyAsync(Id))
            return false;

        var entity = await BaseDal.QueryById(Id);
        ConvertTEditDto2TEntity(editModel, entity);
        BasePoco ent = entity as BasePoco;
        ent.UpdateIP = ip;
        ent.UpdateProg = api;
        CheckOnly(entity, Id);
        return await BaseDal.Update(entity, lstColumns, lstIgnoreColumns, where);
    }
    public async Task<bool> Update(Dictionary<long, TEditDto> editModels)
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);

        List<TEntity> entities = new List<TEntity>();
        foreach (var keyValuePairs in editModels)
        {
            if (keyValuePairs.Value == null || !BaseDal.Any(keyValuePairs.Key))
                continue;

            var entity = await BaseDal.QueryById(keyValuePairs.Key);

            ConvertTEditDto2TEntity(keyValuePairs.Value, entity);
            BasePoco ent = entity as BasePoco;
            ent.UpdateIP = ip;
            ent.UpdateProg = api;
            CheckOnly(entity, keyValuePairs.Key);
            entities.Add(entity);
        }

        return await BaseDal.Update(entities);
    }
    /// <summary>
    /// 更新实体数据
    /// </summary>
    /// <param name="entity">实体类</param>
    /// <returns></returns>
    public async Task<bool> Update(List<TEntity> listEntity)
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);
        listEntity.ForEach(entity =>
        {
            BasePoco ent = entity as BasePoco;
            ent.UpdateIP = ip;
            ent.UpdateProg = api;
        });
        return await BaseDal.Update(listEntity);
    }

    public async Task<bool> Update(TEntity entity, string where)
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);
        BasePoco ent = entity as BasePoco;
        ent.UpdateIP = ip;
        ent.UpdateProg = api;
        return await BaseDal.Update(entity, where);
    }

    public async Task<bool> Update(object operateAnonymousObjects)
    {
        return await BaseDal.Update(operateAnonymousObjects);
    }

    public async Task<bool> Update(TEntity entity, List<string> lstColumns = null, List<string> lstIgnoreColumns = null, string where = "")
    {
        return await BaseDal.Update(entity, lstColumns, lstIgnoreColumns, where);
    }
    public async Task<bool> Update(List<TEntity> entitys, List<string> lstColumns = null, List<string> lstIgnoreColumns = null, string where = "")
    {
        return await BaseDal.Update(entitys, lstColumns, lstIgnoreColumns, where);
    }


    /// <summary>
    /// 根据实体删除一条数据
    /// </summary>
    /// <param name="entity">实体类</param>
    /// <returns></returns>
    public async Task<bool> Delete(TEntity entity)
    {
        return await BaseDal.Delete(entity);
    }

    /// <summary>
    /// 删除指定ID的数据
    /// </summary>
    /// <param name="id">主键ID</param>
    /// <returns></returns>
    public async Task<bool> DeleteById(object id)
    {
        return await BaseDal.DeleteById(id);
    }

    /// <summary>
    /// 删除指定ID的数据
    /// </summary>
    /// <param name="id">主键ID</param>
    /// <returns></returns>
    public virtual async Task<bool> DeleteById1(object id)
    {
        var entity = await BaseDal.QueryById(id);
        BasePoco ent = entity as BasePoco;
        ent.IsEnable = 0;
        return await BaseDal.Update(entity);
    }

    /// <summary>
    /// 删除指定ID集合的数据(批量删除)
    /// </summary>
    /// <param name="ids">主键ID集合</param>
    /// <returns></returns>
    public async Task<bool> DeleteByIds(object[] ids)
    {
        return await BaseDal.DeleteByIds(ids);
    }
    /// <summary>
    /// 删除指定ID集合的数据(批量删除)
    /// </summary>
    /// <param name="ids">主键ID集合</param>
    /// <returns></returns>
    public virtual async Task<bool> DeleteByIds1(long[] ids)
    {
        HttpRequest request = UserContext.Context.Request;
        var api = request.Path.ObjToString().TrimEnd('/').ToLower();
        var ip = GetUserIp(UserContext.Context);

        List<TEntity> entities = new List<TEntity>();
        foreach (var id in ids)
        {
            if (id == null || !BaseDal.Any(id))
                continue;

            var entity = await BaseDal.QueryById(id);

            BasePoco ent = entity as BasePoco;
            ent.UpdateIP = ip;
            ent.UpdateProg = api;
            ent.IsEnable = 0;
            entities.Add(entity);
        }

        return await BaseDal.Update(entities);
    }

    /// <summary>
    /// 根据表达式，删除实体
    /// </summary>
    /// <param name="whereExpression">表达式</param>
    /// <returns></returns>
    public async Task<bool> Delete(Expression<Func<TEntity, bool>> whereExpression)
    {
        return await BaseDal.Delete(whereExpression);
    }

    /// <summary>
    /// 查询所有数据
    /// </summary>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query()
    {
        return await BaseDal.Query();
    }

    /// <summary>
    /// 查询数据列表
    /// </summary>
    /// <param name="where">条件</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(string where)
    {
        return await BaseDal.Query(where);
    }

    /// <summary>
    /// 查询数据列表
    /// </summary>
    /// <param name="whereExpression">whereExpression</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(Expression<Func<TEntity, bool>> whereExpression)
    {
        return await BaseDal.Query(whereExpression);
    }

    public virtual async Task<List<TEntityDto>> QueryDto(Expression<Func<TEntity, bool>> whereExpression)
    {
        var data = await BaseDal.Query(whereExpression);
        return Mapper.Map(data).ToANew<List<TEntityDto>>();
    }
    public virtual async Task<TEntity> QuerySingle(object objId)
    {
        var entity = await BaseDal.QueryById(objId);
        return entity;
    }
    public async Task<TEntity> QuerySingle(Expression<Func<TEntity, bool>> whereExpression)
    {
        var entitys = await BaseDal.Query(whereExpression);
        var entity = entitys.FirstOrDefault();
        if (entity == null)
            return default;
        else return entity;
    }
    public virtual async Task<TEntityDto> QuerySingleDto(Expression<Func<TEntity, bool>> whereExpression)
    {
        var entity = await QuerySingle(whereExpression);
        if (entity == null)
            return default;
        else return Mapper.Map(entity).ToANew<TEntityDto>();
    }

    /// <summary>
    /// 按照特定列查询数据列表
    /// </summary>
    /// <typeparam name="TResult"></typeparam>
    /// <param name="expression"></param>
    /// <returns></returns>
    public async Task<List<TResult>> Query<TResult>(Expression<Func<TEntity, TResult>> expression)
    {
        return await BaseDal.Query(expression);
    }

    /// <summary>
    /// 按照特定列查询数据列表带条件排序
    /// </summary>
    /// <typeparam name="TResult"></typeparam>
    /// <param name="whereExpression">过滤条件</param>
    /// <param name="expression">查询实体条件</param>
    /// <param name="orderByFileds">排序条件</param>
    /// <returns></returns>
    public async Task<List<TResult>> Query<TResult>(Expression<Func<TEntity, TResult>> expression, Expression<Func<TEntity, bool>> whereExpression, string orderByFileds)
    {
        return await BaseDal.Query(expression, whereExpression, orderByFileds);
    }

    /// <summary>
    /// 查询一个列表
    /// </summary>
    /// <param name="whereExpression">条件表达式</param>
    /// <param name="strOrderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(Expression<Func<TEntity, bool>> whereExpression, Expression<Func<TEntity, object>> orderByExpression, bool isAsc = true)
    {
        return await BaseDal.Query(whereExpression, orderByExpression, isAsc);
    }

    public async Task<List<TEntity>> Query(Expression<Func<TEntity, bool>> whereExpression, string orderByFileds)
    {
        return await BaseDal.Query(whereExpression, orderByFileds);
    }

    /// <summary>
    /// 查询一个列表
    /// </summary>
    /// <param name="where">条件</param>
    /// <param name="orderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(string where, string orderByFileds)
    {
        return await BaseDal.Query(where, orderByFileds);
    }

    /// <summary>
    /// 根据sql语句查询
    /// </summary>
    /// <param name="sql">完整的sql语句</param>
    /// <param name="parameters">参数</param>
    /// <returns>泛型集合</returns>
    public async Task<List<TEntity>> QuerySql(string sql, SugarParameter[] parameters = null)
    {
        return await BaseDal.QuerySql(sql, parameters);
    }

    /// <summary>
    /// 根据sql语句查询
    /// </summary>
    /// <param name="sql">完整的sql语句</param>
    /// <param name="parameters">参数</param>
    /// <returns>DataTable</returns>
    public async Task<DataTable> QueryTable(string sql, SugarParameter[] parameters = null)
    {
        return await BaseDal.QueryTable(sql, parameters);
    }

    /// <summary>
    /// 查询前N条数据
    /// </summary>
    /// <param name="whereExpression">条件表达式</param>
    /// <param name="top">前N条</param>
    /// <param name="orderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(Expression<Func<TEntity, bool>> whereExpression, int top, string orderByFileds)
    {
        return await BaseDal.Query(whereExpression, top, orderByFileds);
    }

    /// <summary>
    /// 查询前N条数据
    /// </summary>
    /// <param name="where">条件</param>
    /// <param name="top">前N条</param>
    /// <param name="orderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(string where, int top, string orderByFileds)
    {
        return await BaseDal.Query(where, top, orderByFileds);
    }

    /// <summary>
    /// 分页查询
    /// </summary>
    /// <param name="whereExpression">条件表达式</param>
    /// <param name="pageIndex">页码（下标0）</param>
    /// <param name="pageSize">页大小</param>
    /// <param name="orderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(Expression<Func<TEntity, bool>> whereExpression, int pageIndex, int pageSize, string orderByFileds)
    {
        return await BaseDal.Query(whereExpression, pageIndex, pageSize, orderByFileds);
    }

    /// <summary>
    /// 分页查询
    /// </summary>
    /// <param name="where">条件</param>
    /// <param name="pageIndex">页码（下标0）</param>
    /// <param name="pageSize">页大小</param>
    /// <param name="orderByFileds">排序字段，如name asc,age desc</param>
    /// <returns>数据列表</returns>
    public async Task<List<TEntity>> Query(string where, int pageIndex, int pageSize, string orderByFileds)
    {
        return await BaseDal.Query(where, pageIndex, pageSize, orderByFileds);
    }

    public async Task<PageModel<TEntity>> QueryPage(Expression<Func<TEntity, bool>> whereExpression, int pageIndex = 1, int pageSize = 20, string orderByFileds = null)
    {
        return await BaseDal.QueryPage(whereExpression, pageIndex, pageSize, orderByFileds);
    }

    public virtual async Task<ServicePageResult<TEntityDto>> QueryFilterPage(QueryBody filter)
    {
        return await QueryFilterPage(filter, null);
    }
    public virtual async Task<ServicePageResult<TEntityDto>> QueryFilterPage(QueryBody filter, string condition, bool? IsEnable = true)
    {
        if (string.IsNullOrWhiteSpace(filter.orderBy))
            filter.orderBy = "CreateTime1 DESC";

        if (filter.pageSize == 0)
            filter.pageSize = 10000;

        Type entityType = typeof(TEntity);
        var properties = entityType.GetGenericProperties();

        var countSql = @$" SELECT COUNT(1) FROM  {entityType.GetEntityTableName()} A";
        var sql1 = @$"DECLARE @langId   INT = {filter.langId};";
        var sql = @$" SELECT A.*,
                           ISNULL ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                                    FROM Ghrs_User B
                                    WHERE B.UserId = A.CreateBy),
                                   '') CreateDataInfo,
                           ISNULL ((SELECT CASE WHEN @langId = 1 THEN UserName ELSE UserEname END
                                    FROM Ghrs_User B
                                    WHERE B.UserId = A.UpdateBy),
                                   '') UpdateDataInfo, ISNULL(A.UpdateTime, A.CreateTime) CreateTime1
                    FROM {entityType.GetEntityTableName()} A";
        string conditions = " WHERE 1=1 ";

        if (properties.Any(x => x.Name == "DataBelongID"))
            conditions += $" AND dbo.FUserDataBelongPriv ({App.User.ID}, A.DataBelongID, NULL) = 1";

        if (IsEnable == true)
            conditions += " AND IsEnable = 1";
        else if (IsEnable == false)
            conditions += " AND IsEnable = 0";

        if (!string.IsNullOrWhiteSpace(condition))
            conditions += " AND " + condition;


        if (filter.jsonParam != null)
            foreach (JProperty jProperty in filter.jsonParam.Properties())
            {
                var name = jProperty.Name;
                var value = jProperty.Value.ToString();
                if (name == "page" || name == "pageSize" || !properties.Any(x => x.Name == name))
                    continue;
                if (!string.IsNullOrWhiteSpace(value))
                    conditions = DealConditions(conditions, name, value);
            }

        sql += conditions;
        countSql += conditions;
        int total = await Db.Ado.GetIntAsync(countSql);

        sql = "SELECT * FROM (SELECT *, ROW_NUMBER() OVER (ORDER BY " + filter.orderBy + ") NUM FROM (SELECT * FROM (" + sql + " ";
        sql += ") A ) B ) C";

        sql += " WHERE NUM <= " + filter.pageNum * filter.pageSize + " AND NUM >" + (filter.pageNum - 1) * filter.pageSize;
        sql = sql1 + sql;
        var entitys = await Db.Ado.SqlQueryAsync<TEntityDto>(sql);

        if (entitys != null)
        {
            var dyDataBelongID = entityType.GetProperty("DataBelongID");
            var entityTypeDto = typeof(TEntityDto);
            var propertieDtos = entityTypeDto.GetGenericProperties();
            if (propertieDtos.Any(x => x.Name == "DataBelongName"))
                for (int i = 0; i < entitys.Count; i++)
                {
                    var dataBelongID = dyDataBelongID.GetValue(entitys[i]);
                    if (dataBelongID != null)
                    {
                        var propertie = entityTypeDto.GetProperty("DataBelongName");
                        propertie.SetValue(entitys[i], await GetDataBelongName(dataBelongID.ObjToInt()));
                    }
                }
        }

        return new ServicePageResult<TEntityDto>(filter.pageNum, total, filter.pageSize, entitys);

    }
    public virtual async Task<ServiceResult<ExcelData>> ExportExcel(QueryExport body)
    {
        QueryBody filter = new QueryBody();
        filter.pageNum = 1;
        filter.jsonParam = body.jsonParam;
        filter.pageSize = 1000000;
        filter.langId = body.langId;

        var condition = "1=1";
        if (body.exportSet.SelectRowKeys != null && body.exportSet.SelectRowKeys.Any())
            condition += $" AND Id IN({string.Join(",", body.exportSet.SelectRowKeys)})";

        var data = await QueryFilterPage(filter, condition, true);

        string sql = $@"SELECT *
                            FROM Ghrs_PageSettingQuery
                            WHERE     IsEnable = 1
                                  AND PageNo = '{body.menuName}'
                                  AND (defaultHidden = 'false' OR defaultHidden is null OR defaultHidden ='')
                            ORDER BY SortNo ASC";

        var columns = DbAccess.QueryList<QueryExportColumn>(sql);

        var fieldDescs = new Dictionary<string, string>();
        if (body.exportSet.ExFields.Any())
            body.exportSet.ExFields.ForEach(x =>
            {
                if (columns.Any(o => o.field == x))
                {
                    var label = columns.FirstOrDefault(o => o.field == x)?.label;
                    if (!fieldDescs.ContainsKey(x))
                        fieldDescs.Add(x, label);

                }
            });
        else
            fieldDescs = columns.ToDictionary(item => item.field, item => item.label);
        var dt = ToDataTable(data.result.DT_TableDataT1, fieldDescs, null);
        // 获取所有列名
        var dtColumns = dt.Columns;

        var id = SnowFlakeSingle.instance.getID();
        var physicsPath = $"{Environment.CurrentDirectory}{Path.DirectorySeparatorChar}wwwroot";
        var path = $"{$"{Path.DirectorySeparatorChar}files{Path.DirectorySeparatorChar}export{Path.DirectorySeparatorChar}{id}{Path.DirectorySeparatorChar}"}";
        if (!Directory.Exists(physicsPath + path))
            Directory.CreateDirectory(physicsPath + path);

        path = path + body.exportSet.TitleName + ".xlsx";
        NPOIHelper.ExportExcel(dt, body.exportSet.TitleName, "sheet1", physicsPath + path);

        var result = new ExcelData();
        result.filePath = "/Advanced" + path;
        result.fileName = body.exportSet.TitleName + ".xlsx";
        return ServiceResult<ExcelData>.OprateSuccess("导出成功", result);
    }
    /// <summary>
    /// 列名按照前端显示顺序导出
    /// </summary>
    /// <param name="columns"></param>
    /// <param name="ExportFields"></param>
    /// <returns></returns>
    public static (List<string>, List<string>) Sort(Dictionary<string, string> columns, List<string> ExportFields)
    {
        List<string> fields = new List<string>();
        List<string> colunms = new List<string>();
        if (ExportFields == null || ExportFields.Count == 0)
        {
            return (columns.Keys.ToList(), columns.Values.ToList());
        }

        foreach (var field in ExportFields)
        {
            foreach (var item in columns)
            {
                if (item.Key == field)
                {
                    fields.Add(item.Key);
                    colunms.Add(item.Value);
                }
            }
        }

        return (fields, colunms);
    }
    public static DataTable ToDataTable(List<TEntityDto> list, Dictionary<string, string> fieldDescs = null, params string[] propertyName)
    {
        var (fields, colunms) = Sort(fieldDescs, null);

        List<string> propertyNameList = new List<string>();
        if (propertyName != null)
        {
            propertyNameList.AddRange(propertyName);
        }
        DataTable result = new DataTable();
        if (list.Count > 0)
        {
            PropertyInfo[] propertys = list[0].GetType().GetProperties();
            for (int i = 0; i < fields.Count; i++)
            {
                foreach (PropertyInfo pi in propertys)
                {
                    if (propertyNameList.Count == 0)
                    {
                        //if (DBNull.Value.Equals(pi.PropertyType))
                        //{
                        //   // pi.PropertyType = DateTime;
                        //}
                        Type colType = pi.PropertyType;
                        if (colType.IsGenericType && colType.GetGenericTypeDefinition() == typeof(Nullable<>))
                        {
                            colType = colType.GetGenericArguments()[0];
                        }
                        if (fields[i] == pi.Name)
                        {
                            result.Columns.Add(colunms[i], colType);
                        }
                        //result.Columns.Add(pi.Name, pi.PropertyType);
                    }
                    else
                    {
                        if (propertyNameList.Contains(pi.Name))
                        {
                            if (fields[i] == pi.Name)
                                result.Columns.Add(fields[i], pi.PropertyType);
                        }
                    }
                }
            }
            for (int i = 0; i < list.Count; i++)
            {
                ArrayList tempList = new ArrayList();
                for (int j = 0; j < fields.Count; j++)
                {
                    foreach (PropertyInfo pi in propertys)
                    {
                        if (fields[j] == pi.Name)
                        {
                            if (propertyNameList.Count == 0)
                            {
                                object obj = pi.GetValue(list[i], null);
                                tempList.Add(obj);
                            }
                            else
                            {
                                if (propertyNameList.Contains(pi.Name))
                                {
                                    object obj = pi.GetValue(list[i], null);
                                    tempList.Add(obj);
                                }
                            }
                        }
                    }
                }
                object[] array = tempList.ToArray();

                try
                {
                    result.LoadDataRow(array, true);
                }
                catch (Exception E)
                {

                }

            }
        }
        return result;
    }
    public virtual async Task<ServiceResult<ExcelData>> ImportExcel(IFormFile file, string menuName = null, long? MasterId = null)
    {
        var data = new ExcelData();
        var (path, filepath) = await ReportHelper.GetImportFilePath(file, menuName);
        var id1 = SnowFlakeSingle.instance.getID();
        string errorFileName = path + SnowFlakeSingle.instance.getID() + FileHelper.GetPostfixStr(filepath);

        var dt = ReportHelper.ReadImportExcel(filepath);
        var columns = await QueryExportColumn(menuName);

        var (dictList, errorCount) = await ReportHelper.ValidImportExcel(Db, columns, dt);

        if (errorCount > 0)
        {
            NPOIHelper.ExportExcel(dt, null, "导入数据", FileHelper.GetPhysicsPath() + errorFileName);
            data.filePath = "/Advanced" + errorFileName;
            data.ErrorCount = errorCount;
        }
        else
        {
            Type entityType = typeof(TEntity);
            await Db.Insertable(dictList).AS(entityType.GetEntityTableName()).ExecuteCommandAsync();
            data.SuccessCount = dictList.Count;
        }

        return ServiceResult<ExcelData>.OprateSuccess("导入成功！", data);
    }
    public async Task<List<QueryExportColumn>> QueryExportColumn(string menuName)
    {
        var sql = $@"SELECT Row_Number ()
                              OVER (ORDER BY CONVERT (INT, rowNum), CONVERT (INT, colNum))
                              sortNum,
                           field,
                           [dbo].[FLangKeyToValue] (mkey, 1, label)
                              label,
                           dbo.FS_GetdataSourceBySet
                           (dataSource, APIDataSourceType, Ghrs_PageSettingEdit.APIDataSourceID)
                              dataSource,
                           APIDataSourceType
                              dataSourceType,
                           required,
                           dataType,
                           CONVERT (NVARCHAR (1000), '')
                              commentText                                             -- StaffWith
                                         ,
                             elementType
                           + CASE WHEN multipleSelect = 'true' THEN '_multiple' ELSE '' END
                              elementType,
                           APIDataSourceID
                              dataSourceId,
                           CASE
                              WHEN elementType = 'YearMonthPicker' AND required = 'true'
                              THEN
                                 '必填;输入年月,如(2024-01)，不是年月日'
                              WHEN elementType = 'YearMonthPicker'
                              THEN
                                 '输入年月,如(2024-01)，不是年月日'
                              WHEN required = 'true'
                              THEN
                                 '必填'
                              WHEN required = 'true' AND dataType = 'int'
                              THEN
                                 '必填且输入整数'
                              WHEN required = 'true' AND dataType = 'decimal'
                              THEN
                                 '必填且输入整数'
                              WHEN required = 'false' AND dataType = 'int'
                              THEN
                                 '输入整数'
                              WHEN elementType = 'GDatePicker'
                              THEN
                                 '输入日期'
                              ELSE
                                 NULL
                           END
                              Comment
                    FROM Ghrs_PageSettingEdit
                    WHERE     IsEnable = 1
                          AND elementType NOT IN ('FnKey', 'PageGroup')
                          AND pageNo = '{menuName}'
                          --and editable = 'true'
                          AND defaultHidden ! = 'true'
                          AND elementType ! = 'FileUpload'
                          AND dataType ! = ''";

        return await Db.Ado.SqlQueryAsync<QueryExportColumn>(sql);
    }

    public virtual async Task<ServiceResult<string>> DownloadExcel(string menuName)
    {

        var tableNmae = string.Empty;
        Type entityType = typeof(TEntity);
        var columns = await QueryExportColumn(menuName);

        var dt = await Db.Ado.GetDataTableAsync("SELECT TOP 1 * FROM " + entityType.GetEntityTableName() + " WHERE IsEnable=1");

        var MenuName = await Db.Ado.GetStringAsync($"SELECT MenuName FROM Ghrs_Menu WHERE MenuNo = '{menuName}'");
        //columns = columns.WhereIF(param.exportSet.ExFields.Any(), x => param.exportSet.ExFields.Contains(x.field)).ToList();

        var path = ReportHelper.ImportTemplate(Db, columns, dt, menuName);
        var result = ServiceResult<string>.OprateSuccess($"{MenuName}_{DateTimeHelper.ConvertToSecondString1(DateTime.Now)}.xlsx", path);
        return result;
    }

    public async Task<List<TResult>> QueryMuch<T, T2, T3, TResult>(Expression<Func<T, T2, T3, object[]>> joinExpression, Expression<Func<T, T2, T3, TResult>> selectExpression, Expression<Func<T, T2, T3, bool>> whereLambda = null) where T : class, new()
    {
        return await BaseDal.QueryMuch(joinExpression, selectExpression, whereLambda);
    }

    public async Task<PageModel<TEntity>> QueryPage(PaginationModel pagination)
    {
        var express = DynamicLinqFactory.CreateLambda<TEntity>(pagination.Conditions);
        return await QueryPage(express, pagination.PageIndex, pagination.PageSize, pagination.OrderByFileds);
    }

    #region 分表

    public async Task<List<long>> AddSplit(TEntity entity)
    {
        return await BaseDal.AddSplit(entity);
    }

    public async Task<bool> UpdateSplit(TEntity entity, DateTime dateTime)
    {
        return await BaseDal.UpdateSplit(entity, dateTime);
    }

    /// <summary>
    /// 根据实体删除一条数据
    /// </summary>
    /// <param name="entity">实体类</param>
    /// <returns></returns>
    public async Task<bool> DeleteSplit(TEntity entity, DateTime dateTime)
    {
        return await BaseDal.DeleteSplit(entity, dateTime);
    }

    public async Task<TEntity> QueryByIdSplit(object objId)
    {
        return await BaseDal.QueryByIdSplit(objId);
    }

    public async Task<PageModel<TEntity>> QueryPageSplit(Expression<Func<TEntity, bool>> whereExpression, DateTime beginTime, DateTime endTime, int pageIndex = 1, int pageSize = 20, string orderByFields = null)
    {
        return await BaseDal.QueryPageSplit(whereExpression, beginTime, endTime, pageIndex, pageSize, orderByFields);
    }

    #endregion

    public static string GetUserIp(HttpContext context)
    {
        string realIP = null;
        string forwarded = null;
        string remoteIpAddress = context.Connection.RemoteIpAddress.ToString();
        if (context.Request.Headers.ContainsKey("X-Real-IP"))
        {
            realIP = context.Request.Headers["X-Real-IP"].ToString();
            if (realIP != remoteIpAddress)
            {
                remoteIpAddress = realIP;
            }
        }
        if (context.Request.Headers.ContainsKey("X-Forwarded-For"))
        {
            forwarded = context.Request.Headers["X-Forwarded-For"].ToString();
            if (forwarded != remoteIpAddress)
            {
                remoteIpAddress = forwarded;
            }
        }
        remoteIpAddress = remoteIpAddress.Replace("::ffff:", null);
        return remoteIpAddress;
    }


    #region 辅助方法
    /// <summary>
    /// 转换TEditDto2TEntity
    /// </summary>
    /// <param name="pTargetObjSrc"></param>
    /// <param name="pTargetObjDest"></param>
    /// <returns></returns>
    protected void ConvertTEditDto2TEntity(TEditDto source, TEntity dest)
    {
        foreach (System.Reflection.PropertyInfo mItem in typeof(TEditDto).GetProperties())
        {
            if (dest.HasField(mItem.Name))
                dest.SetValueForField(mItem.Name, mItem.GetValue(source, null));
        }
        //dest.SetValueForField(DbConsts.ColunmName_LastModificationTime, DateTimeHelper.Now());
        //if (_currentUserId != default)
        //{
        //    //dest.SetValueForField(DbConsts.ColunmName_LastModifierId, _currentUserId);
        //    dest.SetValueForField(DbConsts.ColunmName_LastModifier, _currentUserName);
        //}

        //if (_currentTenantId != null)
        //{
        //    dest.SetValueForField(DbConsts.ColunmName_TenantId, _currentTenantId);
        //}
    }
    protected void ConvertTInsertDto2TEntity(TInsertDto source, TEntity dest)
    {
        foreach (System.Reflection.PropertyInfo mItem in typeof(TEditDto).GetProperties())
        {
            if (dest.HasField(mItem.Name))
                dest.SetValueForField(mItem.Name, mItem.GetValue(source, null));
        }
        //dest.SetValueForField(DbConsts.ColunmName_LastModificationTime, DateTimeHelper.Now());
        //if (_currentUserId != default)
        //{
        //    //dest.SetValueForField(DbConsts.ColunmName_LastModifierId, _currentUserId);
        //    dest.SetValueForField(DbConsts.ColunmName_LastModifier, _currentUserName);
        //}

        //if (_currentTenantId != null)
        //{
        //    dest.SetValueForField(DbConsts.ColunmName_TenantId, _currentTenantId);
        //}
    }

    #region 检查表中是否已经存在相同代码的数据

    public static void CheckOnly(TEntity entity, long? id = null)
    {
        Type entityType = typeof(TEntity);
        var tableName = entityType.GetEntityTableName();

        var names = new List<string>();
        var values = new List<object>();
        var descriptions = new List<string>();
        entity.GetOnlyList(out names, out values, out descriptions);
        for (int i = 0; i < names.Count; i++)
        {
            CheckCodeExist(tableName, names[i], values[i], id != null ? ModifyType.Edit : ModifyType.Add, descriptions[i], id);
        }
    }
    /// <summary>
    /// 检查表中是否已经存在相同代码的数据
    /// </summary>
    /// <param name="tableName">表名</param>
    /// <param name="fieldName">字段名</param>
    /// <param name="fieldValue">字段值</param>
    /// <param name="modifyType">ModifyType.Add,ModifyType.Edit</param>
    /// <param name="rowid">ModifyType.Edit时修改记录的ROW_ID值</param>
    /// <param name="promptName">判断栏位的提示名称</param>
    public static void CheckCodeExist(string tableName, string fieldName, object fieldValue, ModifyType modifyType, string promptName, long? rowid = null, string whereCondition = null)
    {
        try
        {
            CheckCodeExist(tableName, fieldName, fieldValue, modifyType, rowid, promptName, whereCondition);
        }
        catch (Exception)
        {
            throw;
        }
    }

    /// <summary>
    /// 检查表中是否已经存在相同代码的数据
    /// </summary>
    /// <param name="tableName">表名</param>
    /// <param name="fieldName">字段名</param>
    /// <param name="fieldValue">字段值</param>
    /// <param name="whereCondition">条件</param>
    /// <param name="modifyType">ModifyType.Add,ModifyType.Edit</param>
    /// <param name="rowid">ModifyType.Edit时修改记录的ROW_ID值</param>
    /// <param name="promptName">判断栏位的提示名称</param>
    /// <param name="whereCondition">Where后的条件，如：IS_ALCON='Y'</param>
    public static bool CheckCodeExist(string tableName, string fieldName, object fieldValue, ModifyType modifyType, long? rowid, string promptName, string whereCondition)
    {
        try
        {
            bool result = false;
            if (modifyType == ModifyType.Add)
            {
                string sql = string.Empty;
                sql = "SELECT COUNT(*) FROM " + tableName + " WHERE " + fieldName + "='" + fieldValue + "' AND IsEnable=1";

                if (!string.IsNullOrEmpty(whereCondition))
                    sql += " AND " + whereCondition;

                int count = Convert.ToInt32(DbAccess.ExecuteScalar(sql));
                if (count > 0)
                {
                    result = true;
                    throw new Exception(string.Format("{0}【{1}】已经存在！", promptName, fieldValue));
                }
                else
                    result = false;

            }
            else if (modifyType == ModifyType.Edit)
            {
                string sql = string.Empty;
                sql = "SELECT COUNT(*) FROM " + tableName + " WHERE " + fieldName + "='" + fieldValue + "' AND IsEnable=1 AND ID!='" + rowid.Value + "'";

                if (!string.IsNullOrEmpty(whereCondition))
                    sql += " AND " + whereCondition;

                int count = Convert.ToInt32(DbAccess.ExecuteScalar(sql));
                if (count > 0)
                {
                    result = true;
                    throw new Exception(string.Format("{0}【{1}】已经存在！", promptName, fieldValue));
                }
                else
                    result = false;
            }
            return result;
        }
        catch (Exception)
        {
            throw;
        }
    }
    #endregion

    #region 自动编号
    /// <summary>
    /// 自动编号
    /// </summary>
    /// <param name="tableCode">表名</param>
    /// <param name="columnCode">栏位名</param>
    /// <param name="prefixTemp">前缀</param>
    /// <returns></returns>
    public async Task<string> GenerateContinuousSequence(string tableCode, string columnCode, string prefixTemp, int length = 7, int tempLength = 6)
    {
        try
        {
            string result = string.Empty;
            int sequence;

            #region 查询
            DbSelect dbSelect = new(tableCode + " A", "A", null);
            dbSelect.IsInitDefaultValue = false;

            if (!string.IsNullOrEmpty(prefixTemp))
                dbSelect.Select("MAX(SUBSTRING(A." + columnCode + "," + (prefixTemp.Length + 1).ToString() + "," + tempLength.ToString() + "))");
            else
                dbSelect.Select("MAX(A." + columnCode + ")");
            //}
            //dbSelect.Select("MAX(CONVERT(DECIMAL,SUBSTRING(A.ISSUE_NO," + (prefix.Length + dateString.Length + 1).ToString() + "," + tempLength.ToString() + ")))");
            if (!string.IsNullOrEmpty(prefixTemp))
                dbSelect.Where("SUBSTRING(A." + columnCode + ",1," + (prefixTemp.Length).ToString() + ")", " = ", prefixTemp);
            dbSelect.Where("LEN(A." + columnCode + ")", "=", length);
            string sql = dbSelect.GetSql();
            //await Db.Ado.GetScalarAsync(sql)
            string maxSequence = Convert.ToString(await Db.Ado.GetScalarAsync(sql));
            #endregion
            //tempLength = tempLength - dateString.Length;
            if (string.IsNullOrEmpty(maxSequence))
                result = prefixTemp + Convert.ToString(1).PadLeft(tempLength, '0');
            else
            {
                if (!string.IsNullOrEmpty(prefixTemp))
                {
                    if (int.TryParse(maxSequence, out sequence))
                    {
                        sequence += 1;
                        if (sequence.ToString().Length > tempLength)
                            throw new Exception("自动生成字串长度已经超过设定长度!");
                    }
                    else
                        throw new Exception("表中的数据无法进行自动编号,请联系软件开发商!");
                    result = prefixTemp + sequence.ToString().PadLeft(tempLength, '0');
                }
                else
                {
                    if (int.TryParse(maxSequence, out sequence))
                    {
                        sequence += 1;
                        if (sequence.ToString().Length > length)
                            throw new Exception("自动生成字串长度已经超过设定长度!");
                    }
                    else
                        throw new Exception("表中的数据无法进行自动编号,请联系软件开发商!");
                    result = sequence.ToString().PadLeft(length, '0');
                }
            }
            return result;
        }
        catch (Exception) { throw; }
    }
    #endregion

    #region 获取通用参数设定
    public async Task<string> GetParaLabel(string key, string value)
    {
        string label = value;

        if (value.IsNullOrEmpty())
            return null;
        var list = await _caching.GetAsync<List<Ghrs_ParaDetail>>(key);
        if (list == null || (list != null && !list.Any()))
        {
            string sql = $@"SELECT *
FROM Ghrs_ParaDetail
WHERE     ParaMasterId IN (SELECT ParaMasterId
                          FROM Ghrs_ParaMaster
                          WHERE     ParaMasterNo = '{key}'
                                AND ParaTypeNo IN ('GhrStaff',
                                                   'GHRPara',
                                                   'Train',
                                                   'Resume',
                                                   'GHRAttend','Performance','Survey'))
      AND IsEnable = 1
ORDER BY SortNo ASC";
            list = DbAccess.QueryList<Ghrs_ParaDetail>(sql);
            if (list.Any())
                await _caching.SetAsync(key, list);
        }

        if (list.Any(x => x.ParaDetailNo == value))
            label = list.FirstOrDefault(x => x.ParaDetailNo == value)?.ParaDetailName;
        return label;
    }

    public async Task<List<Ghrs_ParaDetail>> GetParaList(string key)
    {
        if (key.IsNullOrEmpty())
            return null;

        var list = await _caching.GetAsync<List<Ghrs_ParaDetail>>(key);
        if (list == null || (list != null && !list.Any()))
        {
            string sql = $@"SELECT *
FROM Ghrs_ParaDetail
WHERE     ParaMasterId IN (SELECT ParaMasterId
                          FROM Ghrs_ParaMaster
                          WHERE     ParaMasterNo = '{key}'
                                AND ParaTypeNo IN ('GhrStaff',
                                                   'GHRPara',
                                                   'Train',
                                                   'Resume',
                                                   'GHRAttend'))
      AND IsEnable = 1
ORDER BY SortNo ASC";
            list = DbAccess.QueryList<Ghrs_ParaDetail>(sql);
            if (list.Any())
                await _caching.SetAsync(key, list);
        }
        return list;
    }

    public async Task<string> GetParaLabelListString(string ParaTypeNo, string ParaMasterNo, string value)
    {
        string label = value;

        if (value.IsNullOrEmpty())
            return null;

        string sql = $@"SELECT [dbo].[FLangKeyToValue](mkey,1,ParaDetailName) ParaDetailName
FROM Ghrs_ParaDetail
where IsEnable=1
  and ParaTypeNo = '{ParaTypeNo}'
  and ParaMasterNo = '{ParaMasterNo}'
  and ParaDetailNo in (select value from Openjson('{value}'))
ORDER BY SortNo ASC";
        var list = DbAccess.QueryList<Ghrs_ParaDetail>(sql);

        label = string.Join(",", list.Select(o => o.ParaDetailName));
        return label;
    }

    public async Task<string> GetParaLabel(int? id)
    {
        string label = string.Empty;

        if (id.IsNullOrEmpty())
            return null;
        var ParaMasterNo = DbAccess.ExecuteScalar(@$"SELECT ParaMasterNo
FROM Ghrs_ParaDetail
WHERE     ParaDetailId ='{id}'");

        var list = await _caching.GetAsync<List<Ghrs_ParaDetail>>(ParaMasterNo.ObjToString());
        if (list == null || (list != null && !list.Any()))
        {
            string sql = $@"SELECT *
FROM Ghrs_ParaDetail
WHERE     ParaMasterId IN (SELECT ParaMasterId
                          FROM Ghrs_ParaMaster
                          WHERE     ParaMasterNo = '{ParaMasterNo}'
                                AND ParaTypeNo IN ('GhrStaff',
                                                   'GHRPara',
                                                   'Train',
                                                   'Resume',
                                                   'GHRAttend'))
      AND IsEnable = 1
ORDER BY SortNo ASC";
            list = DbAccess.QueryList<Ghrs_ParaDetail>(sql);
            if (list.Any())
                await _caching.SetAsync(ParaMasterNo.ObjToString(), list);
        }

        if (list.Any(x => x.ParaDetailId == id))
            label = list.FirstOrDefault(x => x.ParaDetailId == id)?.ParaDetailName;
        return label;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    public async Task<string> GetDataBelongName(int? id)
    {
        string label = string.Empty;

        if (id == null)
            return null;
        if (_caching != null)
        {
            var list = await _caching.GetAsync<List<Ghrs_DataBelong>>("DataBelong");
            if (list == null || (list != null && !list.Any()))
            {
                list = await Db.Queryable<Ghrs_DataBelong>().ToListAsync();
                if (list.Any())
                    await _caching.SetAsync("DataBelong", list);
            }

            if (list.Any(x => x.DataBelongId == id))
                label = list.FirstOrDefault(x => x.DataBelongId == id)?.DataBelongName;
        }
        else
            label = await Db.Queryable<Ghrs_DataBelong>().Where(x => x.DataBelongId == id).Select(x => x.DataBelongName).FirstAsync();


        return label;
    }
    #endregion

    #region 处理json查询条件
    public string DealConditions(string conditions, string name, string value)
    {
        var jsonParam = JsonHelper.JsonToObj<JsonParam>(value);

        switch (jsonParam.operationKey)
        {
            case "Include":
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} LIKE '%{jsonParam.columnValue}%'";
                break;
            case "NotInclude":
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} NOT LIKE '%{jsonParam.columnValue}%'";
                break;
            case "IsNull":
                conditions += $" AND {name} IS NULL";
                break;
            case "NotNull":
                conditions += $" AND {name} IS NOT NULL";
                break;
            case "Equal":
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} ='{jsonParam.columnValue}'";
                break;
            case "NotEqual":
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} !='{jsonParam.columnValue}'";
                break;
            case "GreaterOrEqual"://大于等于
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} >='{jsonParam.columnValue}'";
                break;
            case "Greater"://大于
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} >'{jsonParam.columnValue}'";
                break;
            case "LessOrEqual"://小于等于
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} <='{jsonParam.columnValue}'";
                break;
            case "Less"://小于
                if (jsonParam.columnValue != null)
                    conditions += $" AND {name} <'{jsonParam.columnValue}'";
                break;
            case "EqualAny":// 
                if (jsonParam.columnValue != null)
                {
                    var ids1 = JsonHelper.JsonToObj<List<string>>(jsonParam.columnValue.ToString());

                    conditions += $" AND {name} IN ({string.Join(",", ids1.Select(id => "'" + id + "'"))})";
                }
                break;
            case "NotEqualAny":// 
                if (jsonParam.columnValue != null)
                {
                    var ids1 = JsonHelper.JsonToObj<List<string>>(jsonParam.columnValue.ToString());

                    conditions += $" AND ({name} NOT IN ({string.Join(",", ids1.Select(id => "'" + id + "'"))}) OR {name} IS NULL)";
                }
                break;
            case "Range":// 
                if (jsonParam.columnValue != null)
                {
                    var ids1 = JsonHelper.JsonToObj<List<string>>(jsonParam.columnValue.ToString());

                    conditions += $" AND ({name}>='{ids1[0]} 00:00:00' AND {name}<='{ids1[1]} 23:59:59')";
                }
                break;
            default:
                break;
        }

        return conditions;
    }
    public string DealConditions1(string conditions, string name, string value)
    {
        return DealConditions(conditions, "A." + name, value);
    }
    #endregion

    public string GetQueryString(string sqlSelect, int? currentPage = null, int? pageSize = null, string sortField = null)
    {
        string queryString = string.Empty;
        if (string.IsNullOrEmpty(sortField))
            queryString = "SELECT * FROM (SELECT *, ROW_NUMBER() OVER (ORDER BY ROW_ID) NUM FROM (SELECT * FROM (" + sqlSelect + " WHERE 1=1 ";
        else
        {
            if (!string.IsNullOrEmpty(sortField))
                queryString = "SELECT * FROM (SELECT *, ROW_NUMBER() OVER (ORDER BY " + sortField + ") NUM FROM (SELECT * FROM (" + sqlSelect + " ";
            else
                queryString = "SELECT * FROM (SELECT *, ROW_NUMBER() OVER (ORDER BY " + sortField + " DESC) NUM FROM (SELECT * FROM (" + sqlSelect + " ";
        }
        queryString += ") A ) B ) C";
        if (currentPage != null && pageSize != null)
            queryString += " WHERE NUM <= " + currentPage * pageSize + " AND NUM >" + (currentPage - 1) * pageSize;
        return queryString;
    }

    public int? GetStaffId()
    {
        int? StaffId = null;
        string sql = $"SELECT UserStaffID FROM Ghrs_User WHERE UserId='{App.User.ID}'";
        string StaffId1 = Convert.ToString(DbAccess.ExecuteScalar(sql));
        if (!StaffId1.IsNull())
            StaffId = Convert.ToInt32(StaffId1);
        return StaffId;
    }
    public async Task<Ghrs_User> GetUser()
    {
        return await Db.Queryable<Ghrs_User>().Where(x => x.UserId == App.User.ID).FirstAsync();
    }

    #region 获取部门列表
    public static void LoopToAppendChildren(List<Ghro_Dept> depts, List<Ghro_Dept> depts1, Ghro_Dept dept)
    {
        var depts2 = depts.Where(x => x.ParentDeptID == dept.DeptID).ToList();
        depts1.AddRange(depts2);
        foreach (var item in depts2)
        {
            LoopToAppendChildren(depts, depts1, item);
        }
    }
    public async Task<List<Ghro_Dept>> GetDeptList(List<Ghro_Dept> depts, int? deptId = null)
    {
        if (!depts.Any())
            depts = await Db.Queryable<Ghro_Dept>().ToListAsync();
        if (!deptId.IsNullOrEmpty())
        {
            var dept = depts.Where(x => x.DeptID == deptId).FirstOrDefault();
            List<Ghro_Dept> depts1 = [dept];

            LoopToAppendChildren(depts, depts1, dept);
            return depts1;
        }
        else return depts;
    }
    #endregion

    public async Task<List<int>> GetUserStaffPrivIds(int userId, int? staffId = null, DateTime? date = null)
    {
        if (date == null) date = DateTime.Now;
        var @CDate = date;
        var @UserID = userId;
        var @StaffID = staffId.IsNullOrEmpty() ? -1 : staffId.Value;
        int @UserStaffID = await Db.Ado.GetIntAsync($"SELECT UserStaffID FROM Ghrs_User WHERE UserId='{userId}'");

        var @CompanySpecCode = string.Empty;
        int @UserDeptID = 0;
        //int @DataRoleType = 0; //--1  员工自身  2 全部人员   3    按设定
        //int @DeptType = 0; //1 按挑选   2 主管(含下阶)   3 主管(不含下阶)   4    部门助理(含下阶)   5   部门助理(不含下阶)  6   所在部门(含下阶)  7    所在部门(不含下阶) 8: 全部部门
        int?
            @DeptID,
            @TitleID,
            @GradeID,
            @DataBelongID,
            @PeriodMasterID,
            @ZoneID,
            @StaffType1;
        var sql = @$"SELECT CompanySpecCode
                        FROM ghro_company
                        WHERE IsEnable = 1";
        @CompanySpecCode = await Db.Ado.GetStringAsync(sql);

        //--读取使用者对应的资料权限群组
        sql = @$"SELECT b.DataRoleID,
                DataRoleType,
                DeptType,
                AllDataBelong,
                AllGrade,
                AllTitle,
                AllZone,
                AllPeriodMaster,
                ALLDL_IDL,
                DataRoleNo
         FROM Ghrs_UserDataRole a, Ghrs_DataRole b
         WHERE     UserID = {@UserID}
               AND a.IsEnable = 1
               AND b.IsEnable = 1
               AND b.DataRoleId = a.DataRoleId
         ORDER BY DataRoleType, DeptType DESC";

        var dataRoles = await Db.Ado.SqlQueryAsync<Ghrs_UserDataRoleExtend>(sql);

        var scr = Db.Queryable<Ghra_Staff>();
        if (@StaffID == -1)
        {
            if (dataRoles.Where(x => x.DataRoleType == 2).Any())// 全部人员权限
                return await scr.Select(x => x.StaffID).ToListAsync();

            var staffIds = new List<int>();
            var depts = new List<Ghro_Dept>();
            depts = await GetDeptList(depts);
            for (int i = 0; i < dataRoles.Count; i++)
            {
                var deptIds = new List<int>();
                var dataRole = dataRoles[i];

                //--2 集团数据归属编号
                var dataBelongIds = new List<int>();
                if (dataRole.@AllDataBelong == 0)
                {
                    dataBelongIds = await Db.Queryable<Ghrs_DataRoleDetail>()
                        .Where(x =>
                        x.IsEnable == 1 &&
                        x.DataTypeID == 2 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();
                }

                //-- 3 职等编号
                var gradeIds = new List<int>();
                if (dataRole.@AllGrade == 0)
                {
                    gradeIds = await Db.Queryable<Ghrs_DataRoleDetail>()
                        .Where(x =>
                        x.IsEnable == 1 &&
                        x.DataTypeID == 3 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();
                }

                //-- 4 职称编号
                var titleIds = new List<int>();
                if (dataRole.@AllTitle == 0)
                {
                    titleIds = await Db.Queryable<Ghrs_DataRoleDetail>()
                        .Where(x =>
                        x.IsEnable == 1 &&
                        x.DataTypeID == 4 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();
                }

                //-- 5 厂区编号
                var zoneIds = new List<int>();
                if (dataRole.@AllZone == 0)
                {
                    zoneIds = await Db.Queryable<Ghrs_DataRoleDetail>()
                        .Where(x =>
                        x.IsEnable == 1 &&
                        x.DataTypeID == 5 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();
                }

                //-- 6 计薪类别编号
                var periodMasterIds = new List<int>();
                if (dataRole.@AllPeriodMaster == 0)
                {
                    periodMasterIds = await Db.Queryable<Ghrs_DataRoleDetail>()
                        .Where(x =>
                        x.IsEnable == 1 &&
                        x.DataTypeID == 6 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();
                }

                //-- 7 直间接编号
                var DL_IDLIds = new List<int>();
                if (dataRole.@ALLDL_IDL == 0)
                    DL_IDLIds = await Db.Queryable<Ghrs_DataRoleDetail>().Where(x =>
                    x.IsEnable == 1 &&
                        x.DataTypeID == 6 &&
                        x.DataID != null &&
                        x.DataRoleId != null &&
                        x.DataRoleId == dataRole.DataRoleID)
                        .Select(x => x.DataID.Value).ToListAsync();

                //判断部门的权限  @DeptType


                //7 所在部门(不含下阶)
                if (dataRole.@DeptType == 7)
                    deptIds.Add(@UserStaffID);

                //6 所在部门(含下阶)
                if (dataRole.@DeptType == 6)
                {
                    var dept = depts.Where(x => x.DeptID == @UserStaffID).FirstOrDefault();
                    if (dept != null)
                        await GetDeptList(depts, dept.DeptID);
                }

                //5 部门助理(不含下阶)
                if (dataRole.@DeptType == 5)
                {
                    var depts1 = depts.Where(x => x.DeptAssID == @UserStaffID).ToList();
                    deptIds.AddRange(depts1.Select(x => x.DeptID.Value).ToList());
                }

                //4 部门助理(含下阶)
                if (dataRole.@DeptType == 4)
                {
                    var depts1 = depts.Where(x => x.DeptAssID == @UserStaffID).ToList();

                    foreach (var dept in depts1)
                    {
                        var items = await GetDeptList(depts, dept.DeptID);
                        deptIds.AddRange(items.Select(x => x.DeptID.Value).ToList());
                    }
                }

                //3 主管(不含下阶)
                if (dataRole.@DeptType == 3)
                {
                    var depts1 = depts.Where(x => x.DeptManagerID == @UserStaffID).ToList();

                    deptIds.AddRange(depts1.Select(x => x.DeptID.Value).ToList());
                }

                //2 主管(含下阶)
                if (dataRole.@DeptType == 2)
                {
                    var depts1 = depts.Where(x => x.DeptManagerID == @UserStaffID).ToList();

                    foreach (var dept in depts1)
                    {
                        var items = await GetDeptList(depts, dept.DeptID);
                        deptIds.AddRange(items.Select(x => x.DeptID.Value).ToList());
                    }
                }

                //1 按挑选
                if (dataRole.@DeptType == 1)
                {

                    sql = @$"SELECT DataID
                            FROM Ghrs_DataRoleDetail
                            WHERE     IsEnable = 1
                                  AND DataRoleID = {dataRole.DataRoleID}
                                  AND DataTypeID = 1
                                  AND IncludeSub = 0";

                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);


                    if (!dataIDs.Any())
                    {
                        sql = @$"SELECT DataID
                  FROM Ghrs_DataRoleDetail
                  WHERE     IsEnable = 1
                        AND DataRoleID = {dataRole.DataRoleID}
                        AND DataTypeID = 1
                        AND IncludeSub = 1";

                        dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);

                        foreach (var dataID in dataIDs)
                        {
                            var items = await GetDeptList(depts, dataID);
                            deptIds.AddRange(items.Select(x => x.DeptID.Value).ToList());
                        }
                    }
                    else
                        deptIds.AddRange(dataIDs);

                }
                deptIds = deptIds.Distinct().ToList();
                var ids = await scr
                    .WhereIF(dataRole.@AllDataBelong == 0, x => x.DataBelongID != null && dataBelongIds.Contains(x.DataBelongID.Value))
                    .WhereIF(dataRole.@AllGrade == 0, x => x.GradeID != null && gradeIds.Contains(x.GradeID.Value))
                    .WhereIF(dataRole.@AllTitle == 0, x => x.TitleID != null && titleIds.Contains(x.TitleID.Value))
                    .WhereIF(dataRole.@AllZone == 0, x => x.ZoneID != null && zoneIds.Contains(x.ZoneID.Value))
                    .WhereIF(dataRole.@AllPeriodMaster == 0, x => x.PeriodMasterID != null && periodMasterIds.Contains(x.PeriodMasterID.Value))
                    .WhereIF(dataRole.@ALLDL_IDL == 0, x => x.StaffType1 != null && DL_IDLIds.Contains(x.StaffType1.Value))
                    .Where(x => (x.DeptID != null && deptIds.Contains(x.DeptID.Value))
                    ).Select(x => x.StaffID).ToListAsync();
                staffIds.AddRange(ids);
            }

            staffIds.Add(UserStaffID);
            staffIds.Distinct().ToList();
            return staffIds;

        }
        else
        {
            #region  员工默认有自己的权限
            if (staffId == @UserStaffID)
                return [@StaffID];
            #endregion

            sql = @$"SELECT DeptID,
                       TitleID,
                       GradeID,
                       ZoneID,
                       DataBelongID,
                       PeriodMasterID,
                       StaffType1
                FROM ghra_Staff
                WHERE StaffID = {staffId}";

            var staff = await Db.Ado.SqlQuerySingleAsync<Ghra_Staff>(sql);
            if (staff.IsNullOrEmpty())
                return [];

            @DeptID = staff.DeptID;
            @TitleID = staff.TitleID;
            @GradeID = staff.GradeID;
            @ZoneID = staff.ZoneID;
            @DataBelongID = staff.DataBelongID;
            @PeriodMasterID = staff.PeriodMasterID;
            @StaffType1 = staff.StaffType1;

            #region 员工默认有自己下属的权限
            sql = @$"select count(1) from Ghro_Dept 
		   where IsEnable=1 and DeptManagerID={@UserStaffID} 
			 and [dbo].[FDeptAisBChild] ({@DeptID},DeptID{@CDate},1)=1 )";

            //--副主管也有权限
            int? @IsOneDeptTowManager = await Db.Ado.GetIntAsync($"SELECT dbo.FS_ParameterValue('IsOneDeptTowManager')");
            if (@IsOneDeptTowManager == 1)
                sql = @$"select count(1) from Ghro_Dept 
		   where IsEnable=1 and DeptManagerID2={@UserStaffID} 
			 and [dbo].[FDeptAisBChild] ({@DeptID},DeptID{@CDate},1)=1 )";
            #endregion

            for (int i = 0; i < dataRoles.Count; i++)
            {
                var dataRole = dataRoles[i];

                if (dataRole.DataRoleType == 2)// 全部人员权限
                    return [@StaffID];

                if (dataRole.DataRoleType == 1 && @UserStaffID == @StaffID) // 员工自身权限
                    return [@StaffID];

                //1 部门编号   
                //2 集团数据归属编号   
                //3 职等编号   
                //4 职称编号   
                //5 厂区编号   
                //6 计薪类别编号  
                //7 直间接编号
                if (dataRole.@AllDataBelong == 0)
                {
                    sql = @$"SELECT DataID
            FROM Ghrs_DataRoleDetail
            WHERE     IsEnable = 1
                  AND DataRoleID = {dataRole.DataRoleID}
                  AND {@DataBelongID} = DataID
                  AND DataTypeID = 2";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }

                if (dataRole.@AllGrade == 0)
                {
                    sql = @$"SELECT DataID
            FROM Ghrs_DataRoleDetail
            WHERE     IsEnable = 1
                  AND DataRoleID = {dataRole.DataRoleID}
                  AND {@DataBelongID} = DataID
                  AND DataTypeID = 3";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }
                if (dataRole.@AllTitle == 0)
                {
                    sql = @$"SELECT DataID
            FROM Ghrs_DataRoleDetail
            WHERE     IsEnable = 1
                  AND DataRoleID = {dataRole.DataRoleID}
                  AND {@DataBelongID} = DataID
                  AND DataTypeID = 4";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }
                if (dataRole.@AllZone == 0)
                {
                    sql = @$"SELECT DataID
            FROM Ghrs_DataRoleDetail
            WHERE     IsEnable = 1
                  AND DataRoleID = {dataRole.DataRoleID}
                  AND {@DataBelongID} = DataID
                  AND DataTypeID = 5";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }
                if (dataRole.@AllPeriodMaster == 0)
                {
                    sql = @$"SELECT DataID
            FROM Ghrs_DataRoleDetail
            WHERE     IsEnable = 1
                  AND DataRoleID = {dataRole.DataRoleID}
                  AND {@DataBelongID} = DataID
                  AND DataTypeID = 6";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }
                if (dataRole.ALLDL_IDL == 0)
                {
                    sql = @$"SELECT DataID 
                        FROM Ghrs_DataRoleDetail
                        WHERE     IsEnable = 1
                                AND DataRoleID = {dataRole.DataRoleID}
                                AND {@DataBelongID} = DataID
                                AND DataTypeID = 7";
                    var dataIDs = await Db.Ado.SqlQueryAsync<int>(sql);
                    if (!dataIDs.Any())
                        continue;
                }
                //判断部门的权限  @DeptType
                //1 按挑选
                //2 主管(含下阶)
                //3 主管(不含下阶)
                //4 部门助理(含下阶)
                //5 部门助理(不含下阶)
                //6 所在部门(含下阶)
                //7 所在部门(不含下阶)

                if (dataRole.@DeptType == 7)
                {
                    if (@UserDeptID == 0)
                    {
                        sql = $@"SELECT DeptID 
                                FROM Ghra_staff
                                WHERE StaffID = {@UserStaffID}";
                        @UserDeptID = await Db.Ado.GetIntAsync(sql);
                    }
                    if (@DeptID != @UserDeptID)
                        continue;
                }

                if (dataRole.@DeptType == 6)
                {
                    if (@UserDeptID == 0)
                    {
                        sql = $@"SELECT DeptID 
                                FROM Ghra_staff
                                WHERE StaffID = {@UserStaffID}";
                        @UserDeptID = await Db.Ado.GetIntAsync(sql);
                    }
                    sql = @$"SELECT [dbo].[FDeptAisBChild] ({@DeptID},
                                             {@UserDeptID},
                                             {@CDate},
                                             1)";

                    if (await Db.Ado.GetIntAsync(sql) == 0)
                        continue;
                }

                if (dataRole.@DeptType == 5)
                {
                    sql = @$"SELECT count (1)
                        FROM Ghro_Dept
                        WHERE     IsEnable = 1
                              AND DeptAssID = {@UserStaffID}
                              AND DeptID = {@DeptID}";

                    if (await Db.Ado.GetIntAsync(sql) == 0)
                        continue;
                }

                if (dataRole.@DeptType == 4)
                {
                    sql = @$"SELECT COUNT (1)
                        FROM Ghro_Dept
                        WHERE     IsEnable = 1
                              AND DeptAssID = {@UserStaffID}
                              AND [dbo].[FDeptAisBChild] ({@DeptID},
                                                          DeptID,
                                                          {@CDate},
                                                          1) = 1";

                    if (await Db.Ado.GetIntAsync(sql) == 0)
                        continue;
                }

                if (dataRole.@DeptType == 3)
                {
                    sql = @$"SELECT count (1)
                FROM Ghro_Dept
                WHERE     IsEnable = 1
                      AND DeptManagerID = {@UserStaffID}
                      AND DeptID = {@DeptID}";

                    if (await Db.Ado.GetIntAsync(sql) == 0)
                        continue;
                }

                if (dataRole.@DeptType == 2)
                {
                    sql = @$"SELECT count (1)
                FROM Ghro_Dept
                WHERE     IsEnable = 1
                      AND DeptManagerID = {@UserStaffID}
                      AND [dbo].[FDeptAisBChild] ({@DeptID},
                                                  DeptID,
                                                  {@CDate},
                                                  1) = 1";

                    if (await Db.Ado.GetIntAsync(sql) == 0)
                        continue;
                }

                if (dataRole.@DeptType == 1)
                {

                    sql = @$"SELECT count (1)
                            FROM Ghrs_DataRoleDetail
                            WHERE     IsEnable = 1
                                  AND DataRoleID = {dataRole.DataRoleID}
                                  AND {@DeptID} = DataID
                                  AND DataTypeID = 1
                                  AND IncludeSub = 0";

                    var tempId = await Db.Ado.GetIntAsync(sql);

                    if (tempId == 0)
                    {
                        sql = @$"SELECT count (1)
                  FROM Ghrs_DataRoleDetail
                  WHERE     IsEnable = 1
                        AND DataRoleID = {dataRole.DataRoleID}
                        AND [dbo].[FDeptAisBChild] ({@DeptID},
                                                    DataID,
                                                    {@CDate},
                                                    1) = 1
                        AND DataTypeID = 1
                        AND IncludeSub = 1";

                        tempId = await Db.Ado.GetIntAsync(sql);
                    }

                    if (tempId == 0) continue;

                }
                return [@StaffID];
            }
        }
        return default;

    }

    //public int GetTotalCount(string sqlSelect)
    //{
    //    string sql = string.Empty;

    //    try
    //    {
    //        int count = Convert.ToInt32(DBHelper.ExecuteScalar(sql));
    //        return count;
    //    }
    //    catch (Exception Ex)
    //    { 
    //        throw;
    //    }
    //}

    public async Task<string> QueryCompanyCode()
    {
        var sql = @$"SELECT CompanySpecCode
                        FROM ghro_company
                        WHERE IsEnable = 1";
        return await Db.Ado.GetStringAsync(sql);
    }

    #region 发送站内信
    public async Task SendMessage(List<int> staffIds, string title, string content, string webUrl = null)
    {
        for (int i = 0; i < staffIds.Count; i++)
        {
            //string sql = @$"select UserId from Ghrs_User where  UserStaffID={staffIds[i]}";

            //var userId = await Db.Ado.GetIntAsync(sql);


            var id = await Db.Insertable(new Ghrm_MessageText()
            {
                GroupType = "Staff",
                Title = title,
                Content = content,
                SortNo = 0,
                IsDefault = 0,
                IsEnable = 1,
                WEBUrl = webUrl,
                Tag = "Resume",
                JumpType = webUrl == null ? "text" : "link"
            }).ExecuteReturnIdentityAsync();
            await Db.Insertable(new Ghrm_MessageUser()
            {
                MessageTextID = id,
                SenderStaffID = 0,
                ReceiveStaffID = staffIds[i],
                SortNo = 0,
                IsDefault = 0,
                IsEnable = 1,
                ReadStatus = 0,
            }).ExecuteReturnIdentityAsync();

        }

        await SendQYWechateMessage(staffIds, title, content);

    }
    public async Task SendQYWechateMessage(List<int> staffIds, string title, string content, string webUrl = null)
    {
        string sql = @$"Declare @Json nvarchar(max) = ''
  set @Json = (select '{title}' title,                
             'textcard' msgtype,                
             '<div class=\""gray\"">'+CONVERT(nvarchar(16),getdate(),121)+'</div> 
                 <div class=\""normal\""> {content}</div>
                 <div class=\""highlight\""> </div>' [description]  ,
             'Re'+(select Top 1 ParameterValue from  Ghrs_Parameter where ParameterNo='GHRURL')+'/appOauth.html?jumpPage='+'F_APP_message/msgGroupList' url,
             'GHR' picurl,                
             '' htmlStr,                
              {string.Join(',', staffIds.Select(x => x))} toStaffIDs   
                 for json path)
 -- select @Json 
  exec [dbo].[PS_CreateWorkwxMsg] @Json,'Resume'";

        sql = @$"Declare @Json nvarchar(max) = ''
  set @Json = (select '{title}' title,                
             'text' msgtype,                
             '{content}\n请至【GHR人力资源系统-招聘】下查看' [description]  ,
             'Re'+(select Top 1 ParameterValue from  Ghrs_Parameter where ParameterNo='GHRURL')+'/appOauth.html?jumpPage='+'F_APP_message/msgGroupList' url,
             'GHR' picurl,                
             '' htmlStr,                
              '{string.Join(',', staffIds.Select(x => x))}' toStaffIDs   
                 for json path)
 -- select @Json 
  exec [dbo].[PS_CreateWorkwxMsg] @Json,'Resume'";

        //        Declare @Json nvarchar(max) = ''
        //  set @Json = (select '测试提醒' title,                
        //             'markdown' msgtype,                
        //             '您有个简历待面试,面试时间：2024/12/15 00:00~08:08，请及时查看！
        //> **流程信息 * *
        //> 申 请 人：< font color =\"info\">tisking 陶兰</font>
        //> 假    别： < font color =\"warning\">事假</font>
        //> 开始时间：< font color =\"comment\">2022-01-21 09:00</font>
        //> 结束时间：< font color =\"comment\">2022-01-21 11:00</font>
        //>
        //> 请尽快核准" 
        //                         ' [description]  ,
        //             '' url,
        //             'GHR' picurl,                
        //             '' htmlStr,                
        //              StaffID toStaffIDs
        //                 from Ghra_Staff
        //                 where StaffNo = 'tisking'
        //                 for json path)
        //            select @Json
        //  exec[dbo].[PS_CreateWorkwxMsg] @Json,'Test'



        await Db.Ado.ExecuteCommandAsync(sql);
    }

    #endregion

    #region 获取多语
    public async Task<string> QueryLangValue(string key, int? langId, string defaultValue)
    {
        string sql = $"select [dbo].[FLangKeyToValue]('{key}',{langId},null)";
        string value = await Db.Ado.GetStringAsync(sql);
        if (value.IsNullOrEmpty()) value = defaultValue;
        return value;
    }
    public async Task<string> QueryLangValue(string key, string defaultValue)
    {
        return await QueryLangValue(key, LangId, defaultValue);
    }
    public async Task<string> QueryLangValue(string key, string defaultValue, object arg0)
    {
        var value = await QueryLangValue(key, LangId, defaultValue);
        return string.Format(value, arg0);
    }
    public async Task<string> QueryLangValue(string key, string defaultValue, object arg0, object arg1)
    {
        var value = await QueryLangValue(key, LangId, defaultValue);
        return string.Format(value, arg0, arg1);
    }
    public async Task<string> QueryLangValue(string key, string defaultValue, object arg0, object arg1, object arg2)
    {
        var value = await QueryLangValue(key, LangId, defaultValue);
        return string.Format(value, arg0, arg1, arg2);
    }
    #endregion



    public async Task<string> QueryConfigValue(string code, string tableName = "Ghre_Config")
    {
        var sql = @$"SELECT ConfigValue FROM {tableName} WHERE ConfigCode='MULTIPLE_EXAM_SCORE_RECOGNITION' AND IsEnable=1";
        return await Db.Ado.GetStringAsync(sql);
    }

    #endregion
}